from os import path, remove
import json
from libs import params_lib as pl
from libs import files_lib as fl
from libs import sa as sal


def leer_config(storage, config_path, config_name):
    config_name = pl.validar_parametros(
        config_name,
        'El nombre del archivo de configuración es obligatorio.')
    ruta_parametros = f"{config_path}/config/{config_name}.local"
    config = fl.leer_json(ruta_parametros)
    local_path = valor_config(config, "local_paths", config_name)
    remote_path = valor_config(config, "storage_paths", config_name)
    config_name = valor_config(config, "files", config_name)
    config = agregar_config(config, "storage_access", json.loads(storage))
    local_file = path.join(local_path, "config",  config_name)
    remote_file = path.join(remote_path, config_name)
    existe = sal.existe_archivo(config, remote_file)
    if existe:
        salida = sal.lee_sa(config, local_file, remote_file)
        config = fl.leer_json(local_file)
        config = agregar_config(config, "storage_access", json.loads(storage))
        remove(local_file)
    else:
        raise Exception(f"No se encontró el archivo config en la ruta: {remote_file}")
    return config


def leer_storage_local(config_path, config_name):
    config_name = pl.validar_parametros(
        config_name,
        'El nombre del archivo de configuración es obligatorio.')
    ruta_parametros = f"{config_path}/config/{config_name}.local"
    config = fl.leer_json(ruta_parametros)
    k = [ "azr_sa_accountName", "azr_sa_accountKey", "azr_sa_bucket" ]
    p0 = valor_config(config, "s3", k[0])
    p1 = valor_config(config, "s3", k[1])
    p2 = valor_config(config, "s3", k[2])
    storage = { k[0]: p0, k[1]: p1, k[2]: p2 }
    return json.dumps(storage)


def leer_oracle_local(config_path, config_name):
    config_name = pl.validar_parametros(
        config_name,
        'El nombre del archivo de configuración es obligatorio.')
    ruta_parametros = f"{config_path}/config/{config_name}.local"
    config = fl.leer_json(ruta_parametros)
    k = [ "host", "port", "sid", "userName", "password" ]
    p1 = valor_config(config, "oracle", k[0])
    p2 = valor_config(config, "oracle", k[1])
    p3 = valor_config(config, "oracle", k[2])
    p4 = valor_config(config, "oracle", k[3])
    p5 = valor_config(config, "oracle", k[4])
    oracle = { k[0]: p1, k[1]: p2, k[2]: p3, k[3]: p4, k[4]: p5 }
    return json.dumps(oracle)


def leer_smtp_local(config_path, config_name):
    config_name = pl.validar_parametros(
        config_name,
        'El nombre del archivo de configuración es obligatorio.')
    ruta_parametros = f"{config_path}/config/{config_name}.local"
    config = fl.leer_json(ruta_parametros)
    k = [ "server", "port", "sender", "proxy", "authentication", "keysender" ]
    p1 = valor_config(config, "smtp", k[0])
    p2 = valor_config(config, "smtp", k[1])
    p3 = valor_config(config, "smtp", k[2])
    p4 = valor_config(config, "smtp", k[3])
    p5 = valor_config(config, "smtp", k[4])
    p6 = valor_config(config, "smtp", k[5])
    smtp = { k[0]: p1, k[1]: p2, k[2]: p3, k[3]: p4, k[4]: p5, k[5]:p6 }
    return json.dumps(smtp)


def leer_key_local(config_path, config_name):
    config_name = pl.validar_parametros(
        config_name,
        'El nombre del archivo de configuración es obligatorio.')
    ruta_parametros = f"{config_path}/config/{config_name}.local"
    config = fl.leer_json(ruta_parametros)
    key = valor_config(config, "key", "value")
    return key


def agregar_config(config, raiz, valor):
    config[raiz] = valor
    return config


def valor_config(config, param1, param2="", param3=""):
    valor = fl.valor_json(config, param1, param2, param3)
    return valor
