# utils
from libs import oracle_lib as ol, config_lib as cl, params_lib as pl, files as fl


def generate_select_sql(schema, nombreTabla, partition, columnas, where):
    sql_select =  f"SELECT {columnas}"
    if partition.strip() != "":
        sql_from = f"FROM {schema}.{nombreTabla} PARTITION({partition})"
    else:
        sql_from = f"FROM {schema}.{nombreTabla}"
    if where.strip() != "":
        sql_where = f"WHERE {where}"
    else:
        sql_where = ""
    sql = f"{sql_select} {sql_from} {sql_where}"
    return sql


def generate_insert_sql(schema, nombreTabla, columnas, cantidadColumnas):
    if columnas == "*":
        sqlColumnas = ""
    else:
        sqlColumnas = f"({columnas})"
    numeracionCampos = [":"+str(i+1) for i in range(cantidadColumnas)]
    numeracionCampos = ",".join(numeracionCampos)
    sql = f"INSERT INTO {schema}.{nombreTabla} {sqlColumnas} values ({numeracionCampos})"

    return sql


def obtener_parametros_servidor(config):
    host = pl.validar_parametros(
        cl.valor_config(config, "oracleconn", "host"),
        "El parametro host de la base de datos es obligatorio.",
    )
    port = pl.validar_parametros(
        cl.valor_config(config, "oracleconn", "port"),
        "El parametro host de la base de datos es obligatorio.",
    )
    sid = pl.validar_parametros(
        cl.valor_config(config, "oracleconn", "sid"),
        "El parametro host de la base de datos es obligatorio.",
    )
    userName = pl.validar_parametros(
        cl.valor_config(config, "oracleconn", "userName"),
        "El parametro host de la base de datos es obligatorio.",
    )
    password = pl.validar_parametros(
        cl.valor_config(config, "oracleconn", "password"),
        "El parametro host de la base de datos es obligatorio.",
    )
    return host, port, sid, userName, password


def existe_tabla(config, schema, nombreTabla):
    host, port, sid, userName, password = obtener_parametros_servidor(config)
    sql_select = f"""
        SELECT COUNT(1) AS CNT
        FROM ALL_OBJECTS
        WHERE OBJECT_TYPE = 'TABLE'
        AND UPPER(OWNER) = '{schema}'
        AND UPPER(OBJECT_NAME) ='{nombreTabla}'
        """
    rows, col_name = ol.select_table(
        host, port, sid, userName, password, sql_select)
    if rows[0][0] == 0:
        raise Exception(f"Tabla {schema}.{nombreTabla} no existe")


def existe_tabla_particion(config, schema, nombreTabla, particion):
    host, port, sid, userName, password = obtener_parametros_servidor(config)
    sql_select = f"""
        SELECT COUNT(1) AS CNT
        FROM ALL_TAB_PARTITIONS  
        WHERE UPPER(TABLE_OWNER) = '{schema}'
        AND UPPER(TABLE_NAME) = '{nombreTabla}'
        AND UPPER(PARTITION_NAME) = '{particion}'
        """
    rows, col_name = ol.select_table(
        host, port, sid, userName, password, sql_select)
    if rows[0][0] == 0:
        raise Exception(f"No existe la partición {particion} para la tabla {schema}.{nombreTabla}.")


def select_tabla(config, schema, nombreTabla, partition, columnas, where):
    host, port, sid, userName, password = obtener_parametros_servidor(config)
    sql_select = generate_select_sql(schema, nombreTabla, partition, columnas, where)
    existe_tabla(config, schema, nombreTabla)
    if partition.strip() != "":
        existe_tabla_particion(config, schema, nombreTabla, partition)
    print(sql_select)
    rows, col_name = ol.select_table(host, port, sid, userName, password, sql_select)
    return rows, col_name


def insertar_tabla(config, tup_insert, schema, nombreTabla, columnas, cantidadColumnas):
    host, port, sid, userName, password = obtener_parametros_servidor(config)
    sql_insert = generate_insert_sql(schema, nombreTabla, columnas, cantidadColumnas)
    existe_tabla(config, schema, nombreTabla)
    print(sql_insert)
    ol.insert_table(userName, password, host, port, sid, sql_insert, tup_insert)


def generate_truncate_sp(schema, nombreTabla, truncate_table, truncate_partition, nombreParticion):
    params = []
    spName = "SYS.TRUNCATE_PARTITION"    
    if truncate_partition:
        params = [schema, nombreTabla, nombreParticion]
    else:
        if truncate_table:
            params = [schema, nombreTabla]            
    return spName, params


def truncate_table_partition(config, schema, nombreTabla, truncate_table, truncate_partition, nombreParticion):
    host, port, sid, userName, password = obtener_parametros_servidor(config)
    spName, params = generate_truncate_sp(schema, nombreTabla, truncate_table, truncate_partition, nombreParticion)
    existe_tabla(config, schema, nombreTabla)
    ol.execute_sp(userName, password, host, port, sid, spName, params)


def truncate_table_oracle(config, periodo, origen, keyTabla):
    print(f"Truncate tabla de oracle: {keyTabla}")
    # print(config)
    nombreParticion = ""
    schema = fl.obtener_ruta2(
        config,
        origen,
        keyTabla,
        "schema",
        f"el schema de la tabla de {keyTabla} es obligatorio"
    )
    nombreTabla = fl.obtener_ruta2(
        config,
        origen,
        keyTabla,
        "name",
        f"el nombre de la tabla de {keyTabla} es obligatorio"
    )
    truncate_table = fl.obtener_ruta2(
        config,
        origen,
        keyTabla,
        "truncate_table",
        f"El valor de truncate_table de la tabla {nombreTabla} es obligatorio"
    )
    truncate_partition = fl.obtener_ruta2(
        config,
        origen,
        keyTabla,
        "truncate_partition",
        f"El valor de truncate_partition de la tabla {nombreTabla} es obligatorio"
    )

    if truncate_partition:
        nombreParticion = fl.obtener_ruta2(
            config,
            origen,
            keyTabla,
            "partition",
            f"El valor de truncate_partition de la tabla {nombreTabla} es obligatorio"
        )
        nombreParticion = nombreParticion.replace("[PERIODO]", periodo)

    if truncate_table | truncate_partition:
        truncate_table_partition(config, schema, nombreTabla, truncate_table, truncate_partition, nombreParticion)
