from importlib.resources import Resource
import os
from xml.dom import NotFoundErr
import boto3
from io import StringIO


def s3Client(
    prm_aws_endpoint,
    prm_aws_access_key_id,
    prm_aws_secret_access_key
):

    s3_client = boto3.client(
        "s3",
        endpoint_url=prm_aws_endpoint,
        aws_access_key_id=prm_aws_access_key_id,
        aws_secret_access_key=prm_aws_secret_access_key
    )
    return s3_client


def readS3(
    prm_aws_endpoint,
    prm_aws_access_key_id,
    prm_aws_secret_access_key,
    prm_aws_s3_bucket,
    localpath,
    referencepath
):
    s3 = s3Client(prm_aws_endpoint,
                prm_aws_access_key_id,
                prm_aws_secret_access_key)
    file = os.path.basename(localpath)
    with open(localpath, 'wb') as data:
        s3.download_fileobj(prm_aws_s3_bucket, referencepath, data)
    if not os.path.exists(localpath):
       raise Exception("no se logro escribir el archivo localmente")
    return file


def uploadS3(
    prm_aws_endpoint,
    prm_aws_access_key_id,
    prm_aws_secret_access_key,
    prm_aws_s3_bucket,
    localpath,
    referencepath
):

    s3 = s3Client(prm_aws_endpoint,
                  prm_aws_access_key_id,
                  prm_aws_secret_access_key)
    s3.upload_file(
        Filename=localpath,
        Bucket=prm_aws_s3_bucket,
        Key=referencepath,
        ExtraArgs={'ACL': 'bucket-owner-full-control'}
    )


def writeS3(
    prm_aws_endpoint,
    prm_aws_access_key_id,
    prm_aws_secret_access_key,
    prm_aws_s3_bucket,
    referencepath, data
):

    s3 = s3Client(prm_aws_endpoint,
                  prm_aws_access_key_id,
                  prm_aws_secret_access_key)
    with StringIO() as buffer:
        data.to_csv(buffer, index=False)
        response = s3.put_object(
            Bucket=prm_aws_s3_bucket, Key=referencepath, Body=buffer.getvalue()
        )
    return response


def listSubDirectoriesS3(
    prm_aws_endpoint,
    prm_aws_access_key_id,
    prm_aws_secret_access_key,
    prm_aws_s3_bucket,
    prefixname
                        ):
    s3 = s3Client(
        prm_aws_endpoint,
        prm_aws_access_key_id,
        prm_aws_secret_access_key
        )
    list = []
    response = s3.list_objects_v2(Bucket=prm_aws_s3_bucket, 
                                  Prefix=prefixname, 
                                  Delimiter='/')
    for content in response.get('CommonPrefixes', []):
        list.append(int(content['Prefix'].split("/")[-2]))
    return list


def listFilesS3(
    prm_aws_endpoint,
    prm_aws_access_key_id,
    prm_aws_secret_access_key,
    prm_aws_s3_bucket,
    prefixname
                        ):
    s3 = s3Client(
        prm_aws_endpoint,
        prm_aws_access_key_id,
        prm_aws_secret_access_key
        )
    list = []
    response = s3.list_objects_v2(Bucket=prm_aws_s3_bucket, 
                                  Prefix=prefixname, 
                                  Delimiter='/')
    for content in response.get('Contents', []):
        if content["Size"]>0:
            list.append(content['Key'].split("/")[-1])
    return list


def find(
    prm_aws_endpoint,
    prm_aws_access_key_id,
    prm_aws_secret_access_key,
    prm_aws_s3_bucket,
    prefixname
    ):
    s3 = boto3.resource(
        's3',
        endpoint_url=prm_aws_endpoint,
        aws_access_key_id=prm_aws_access_key_id,
        aws_secret_access_key=prm_aws_secret_access_key
        )
    try:
        s3.Object(prm_aws_s3_bucket, prefixname).load()
    except Exception:
        return False
    else:
        return True
